"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.default = void 0;
var _lambdaHandler = require("./lambda-handler");
var _routeHandler = require("./route-handler");
var _package = _interopRequireDefault(require("gatsby-adapter-netlify/package.json"));
let _cacheUtils;
async function getCacheUtils() {
  if (_cacheUtils) {
    return _cacheUtils;
  }
  if (process.env.NETLIFY) {
    const CACHE_DIR = `/opt/build/cache`;
    _cacheUtils = (await import(`@netlify/cache-utils`)).bindOpts({
      cacheDir: CACHE_DIR
    });
    return _cacheUtils;
  }
  return undefined;
}
const createNetlifyAdapter = options => {
  return {
    name: `gatsby-adapter-netlify`,
    cache: {
      async restore({
        directories,
        reporter
      }) {
        const utils = await getCacheUtils();
        if (utils) {
          reporter.info(`[gatsby-adapter-netlify] using @netlify/cache-utils restore`);
          return await utils.restore(directories);
        }
        return false;
      },
      async store({
        directories,
        reporter
      }) {
        const utils = await getCacheUtils();
        if (utils) {
          reporter.info(`[gatsby-adapter-netlify] using @netlify/cache-utils save`);
          await utils.save(directories);
        }
      }
    },
    async adapt({
      routesManifest,
      functionsManifest
    }) {
      const {
        lambdasThatUseCaching
      } = await (0, _routeHandler.handleRoutesManifest)(routesManifest);

      // functions handling
      for (const fun of functionsManifest) {
        await (0, _lambdaHandler.prepareFunctionVariants)(fun, lambdasThatUseCaching.get(fun.functionId));
      }
    },
    config: ({
      reporter
    }) => {
      var _packageJson$version;
      reporter.info(`[gatsby-adapter-netlify] version: ${(_packageJson$version = _package.default === null || _package.default === void 0 ? void 0 : _package.default.version) !== null && _packageJson$version !== void 0 ? _packageJson$version : `unknown`}`);
      // excludeDatastoreFromEngineFunction can be enabled either via options or via env var (to preserve handling of env var that existed in Netlify build plugin).
      let excludeDatastoreFromEngineFunction = options === null || options === void 0 ? void 0 : options.excludeDatastoreFromEngineFunction;
      if (typeof excludeDatastoreFromEngineFunction === `undefined` && typeof process.env.GATSBY_EXCLUDE_DATASTORE_FROM_BUNDLE !== `undefined`) {
        excludeDatastoreFromEngineFunction = process.env.GATSBY_EXCLUDE_DATASTORE_FROM_BUNDLE === `true` || process.env.GATSBY_EXCLUDE_DATASTORE_FROM_BUNDLE === `1`;
      }
      if (typeof excludeDatastoreFromEngineFunction === `undefined`) {
        excludeDatastoreFromEngineFunction = false;
      }
      const deployURL = process.env.NETLIFY_LOCAL ? `http://localhost:8888` : process.env.DEPLOY_URL;
      if (excludeDatastoreFromEngineFunction && !deployURL) {
        reporter.warn(`[gatsby-adapter-netlify] excludeDatastoreFromEngineFunction is set to true but no DEPLOY_URL is set. Disabling excludeDatastoreFromEngineFunction.`);
        excludeDatastoreFromEngineFunction = false;
      }
      return {
        excludeDatastoreFromEngineFunction,
        deployURL,
        supports: {
          pathPrefix: false,
          trailingSlash: [`always`]
        },
        pluginsToDisable: [`gatsby-plugin-netlify-cache`, `gatsby-plugin-netlify`]
      };
    }
  };
};
var _default = createNetlifyAdapter;
exports.default = _default;