"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.handleRoutesManifest = handleRoutesManifest;
var _os = require("os");
var _fsExtra = _interopRequireDefault(require("fs-extra"));
const NETLIFY_REDIRECT_KEYWORDS_ALLOWLIST = new Set([`query`, `conditions`, `headers`, `signed`, `edge_handler`]);
const NETLIFY_CONDITIONS_ALLOWLIST = new Set([`language`, `country`]);
const toNetlifyPath = (fromPath, toPath) => {
  // Modifies query parameter redirects, having no effect on other fromPath strings
  const netlifyFromPath = fromPath.replace(/[&?]/, ` `);
  // Modifies wildcard & splat redirects, having no effect on other toPath strings
  const netlifyToPath = toPath.replace(/\*/, `:splat`);
  return [netlifyFromPath, netlifyToPath];
};
const MARKER_START = `# gatsby-adapter-netlify start`;
const MARKER_END = `# gatsby-adapter-netlify end`;
async function injectEntries(fileName, content) {
  await _fsExtra.default.ensureFile(fileName);
  const data = await _fsExtra.default.readFile(fileName, `utf8`);
  const [initial = ``, rest = ``] = data.split(MARKER_START);
  const [, final = ``] = rest.split(MARKER_END);
  const out = [initial === _os.EOL ? `` : initial, initial.endsWith(_os.EOL) ? `` : _os.EOL, MARKER_START, _os.EOL, content, _os.EOL, MARKER_END, final.startsWith(_os.EOL) ? `` : _os.EOL, final === _os.EOL ? `` : final].filter(Boolean).join(``).replace(/# @netlify\/plugin-gatsby redirects start(.|\n|\r)*# @netlify\/plugin-gatsby redirects end/gm, ``).replace(/## Created with gatsby-plugin-netlify(.|\n|\r)*$/gm, ``);
  await _fsExtra.default.outputFile(fileName, out);
}
async function handleRoutesManifest(routesManifest) {
  const lambdasThatUseCaching = new Map();
  let _redirects = ``;
  let _headers = ``;
  for (const route of routesManifest) {
    const fromPath = route.path.replace(/\*.*/, `*`);
    if (route.type === `function`) {
      let functionName = route.functionId;
      if (route.cache) {
        functionName = `${route.functionId}-odb`;
        if (!lambdasThatUseCaching.has(route.functionId)) {
          lambdasThatUseCaching.set(route.functionId, functionName);
        }
      }
      const invocationURL = `/.netlify/${route.cache ? `builders` : `functions`}/${functionName}`;
      _redirects += `${encodeURI(fromPath)}  ${invocationURL}  200\n`;
    } else if (route.type === `redirect`) {
      const {
        status: routeStatus,
        toPath,
        force,
        // TODO: add headers handling
        headers,
        ...rest
      } = route;
      let status = String(routeStatus);
      if (force) {
        status = `${status}!`;
      }
      const [netlifyFromPath, netlifyToPath] = toNetlifyPath(fromPath, toPath);

      // The order of the first 3 parameters is significant.
      // The order for rest params (key-value pairs) is arbitrary.
      const pieces = [netlifyFromPath, netlifyToPath, status];
      for (const [key, value] of Object.entries(rest)) {
        if (NETLIFY_REDIRECT_KEYWORDS_ALLOWLIST.has(key)) {
          if (key === `conditions`) {
            // "conditions" key from Gatsby contains only "language" and "country"
            // which need special transformation to match Netlify _redirects
            // https://www.gatsbyjs.com/docs/reference/config-files/actions/#createRedirect
            if (value && typeof value === `object`) {
              for (const [conditionKey, conditionValueRaw] of Object.entries(value)) {
                if (NETLIFY_CONDITIONS_ALLOWLIST.has(conditionKey)) {
                  const conditionValue = Array.isArray(conditionValueRaw) ? conditionValueRaw.join(`,`) : conditionValueRaw;
                  // Gatsby gives us "country", we want "Country"
                  const conditionName = conditionKey.charAt(0).toUpperCase() + conditionKey.slice(1);
                  pieces.push(`${conditionName}:${conditionValue}`);
                }
              }
            }
          } else {
            pieces.push(`${key}=${value}`);
          }
        }
      }
      _redirects += pieces.join(`  `) + `\n`;
    } else if (route.type === `static`) {
      // regular static asset without dynamic paths will just work, so skipping those
      if (route.path.includes(`:`) || route.path.includes(`*`)) {
        _redirects += `${encodeURI(fromPath)}  ${route.filePath.replace(/^public/, ``)}  200\n`;
      }
      _headers += `${encodeURI(fromPath)}\n${route.headers.reduce((acc, curr) => {
        acc += `  ${curr.key}: ${curr.value}\n`;
        return acc;
      }, ``)}`;
    }
  }
  await injectEntries(`public/_redirects`, _redirects);
  await injectEntries(`public/_headers`, _headers);
  return {
    lambdasThatUseCaching
  };
}